# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/fad_pytorch',
                'doc_host': 'https://drscotthawley.github.io',
                'git_url': 'https://github.com/drscotthawley/fad_pytorch',
                'lib_path': 'fad_pytorch'},
  'syms': { 'fad_pytorch.core': {'fad_pytorch.core.foo': ('core.html#foo', 'fad_pytorch/core.py')},
            'fad_pytorch.fad_embed': { 'fad_pytorch.fad_embed.download_file': ('fad_embed.html#download_file', 'fad_pytorch/fad_embed.py'),
                                       'fad_pytorch.fad_embed.download_if_needed': ( 'fad_embed.html#download_if_needed',
                                                                                     'fad_pytorch/fad_embed.py'),
                                       'fad_pytorch.fad_embed.embed': ('fad_embed.html#embed', 'fad_pytorch/fad_embed.py'),
                                       'fad_pytorch.fad_embed.get_ckpt': ('fad_embed.html#get_ckpt', 'fad_pytorch/fad_embed.py'),
                                       'fad_pytorch.fad_embed.main': ('fad_embed.html#main', 'fad_pytorch/fad_embed.py'),
                                       'fad_pytorch.fad_embed.setup_embedder': ( 'fad_embed.html#setup_embedder',
                                                                                 'fad_pytorch/fad_embed.py')},
            'fad_pytorch.fad_gen': { 'fad_pytorch.fad_gen.gen': ('fad_gen.html#gen', 'fad_pytorch/fad_gen.py'),
                                     'fad_pytorch.fad_gen.main': ('fad_gen.html#main', 'fad_pytorch/fad_gen.py')},
            'fad_pytorch.fad_score': { 'fad_pytorch.fad_score.calc_mu_sigma': ('fad_score.html#calc_mu_sigma', 'fad_pytorch/fad_score.py'),
                                       'fad_pytorch.fad_score.calc_score': ('fad_score.html#calc_score', 'fad_pytorch/fad_score.py'),
                                       'fad_pytorch.fad_score.main': ('fad_score.html#main', 'fad_pytorch/fad_score.py'),
                                       'fad_pytorch.fad_score.read_embeddings': ( 'fad_score.html#read_embeddings',
                                                                                  'fad_pytorch/fad_score.py')},
            'fad_pytorch.pann': { 'fad_pytorch.pann.AttBlock': ('pann.html#attblock', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.AttBlock.__init__': ('pann.html#attblock.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.AttBlock.forward': ('pann.html#attblock.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.AttBlock.init_weights': ('pann.html#attblock.init_weights', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.AttBlock.nonlinear_transform': ( 'pann.html#attblock.nonlinear_transform',
                                                                                     'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn10': ('pann.html#cnn10', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn10.__init__': ('pann.html#cnn10.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn10.forward': ('pann.html#cnn10.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn10.init_weight': ('pann.html#cnn10.init_weight', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14': ('pann.html#cnn14', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14.__init__': ('pann.html#cnn14.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14.forward': ('pann.html#cnn14.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14.init_weight': ('pann.html#cnn14.init_weight', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_16k': ('pann.html#cnn14_16k', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_16k.__init__': ('pann.html#cnn14_16k.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_16k.forward': ('pann.html#cnn14_16k.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_16k.init_weight': ('pann.html#cnn14_16k.init_weight', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_8k': ('pann.html#cnn14_8k', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_8k.__init__': ('pann.html#cnn14_8k.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_8k.forward': ('pann.html#cnn14_8k.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_8k.init_weight': ('pann.html#cnn14_8k.init_weight', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_DecisionLevelAtt': ('pann.html#cnn14_decisionlevelatt', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_DecisionLevelAtt.__init__': ( 'pann.html#cnn14_decisionlevelatt.__init__',
                                                                                        'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_DecisionLevelAtt.forward': ( 'pann.html#cnn14_decisionlevelatt.forward',
                                                                                       'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_DecisionLevelAtt.init_weight': ( 'pann.html#cnn14_decisionlevelatt.init_weight',
                                                                                           'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_DecisionLevelAvg': ('pann.html#cnn14_decisionlevelavg', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_DecisionLevelAvg.__init__': ( 'pann.html#cnn14_decisionlevelavg.__init__',
                                                                                        'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_DecisionLevelAvg.forward': ( 'pann.html#cnn14_decisionlevelavg.forward',
                                                                                       'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_DecisionLevelAvg.init_weight': ( 'pann.html#cnn14_decisionlevelavg.init_weight',
                                                                                           'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_DecisionLevelMax': ('pann.html#cnn14_decisionlevelmax', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_DecisionLevelMax.__init__': ( 'pann.html#cnn14_decisionlevelmax.__init__',
                                                                                        'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_DecisionLevelMax.forward': ( 'pann.html#cnn14_decisionlevelmax.forward',
                                                                                       'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_DecisionLevelMax.init_weight': ( 'pann.html#cnn14_decisionlevelmax.init_weight',
                                                                                           'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_emb128': ('pann.html#cnn14_emb128', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_emb128.__init__': ('pann.html#cnn14_emb128.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_emb128.forward': ('pann.html#cnn14_emb128.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_emb128.init_weight': ( 'pann.html#cnn14_emb128.init_weight',
                                                                                 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_emb32': ('pann.html#cnn14_emb32', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_emb32.__init__': ('pann.html#cnn14_emb32.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_emb32.forward': ('pann.html#cnn14_emb32.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_emb32.init_weight': ('pann.html#cnn14_emb32.init_weight', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_emb512': ('pann.html#cnn14_emb512', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_emb512.__init__': ('pann.html#cnn14_emb512.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_emb512.forward': ('pann.html#cnn14_emb512.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_emb512.init_weight': ( 'pann.html#cnn14_emb512.init_weight',
                                                                                 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_mel128': ('pann.html#cnn14_mel128', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_mel128.__init__': ('pann.html#cnn14_mel128.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_mel128.forward': ('pann.html#cnn14_mel128.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_mel128.init_weight': ( 'pann.html#cnn14_mel128.init_weight',
                                                                                 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_mel32': ('pann.html#cnn14_mel32', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_mel32.__init__': ('pann.html#cnn14_mel32.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_mel32.forward': ('pann.html#cnn14_mel32.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_mel32.init_weight': ('pann.html#cnn14_mel32.init_weight', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_mixup_time_domain': ('pann.html#cnn14_mixup_time_domain', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_mixup_time_domain.__init__': ( 'pann.html#cnn14_mixup_time_domain.__init__',
                                                                                         'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_mixup_time_domain.forward': ( 'pann.html#cnn14_mixup_time_domain.forward',
                                                                                        'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_mixup_time_domain.init_weight': ( 'pann.html#cnn14_mixup_time_domain.init_weight',
                                                                                            'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_no_dropout': ('pann.html#cnn14_no_dropout', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_no_dropout.__init__': ( 'pann.html#cnn14_no_dropout.__init__',
                                                                                  'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_no_dropout.forward': ( 'pann.html#cnn14_no_dropout.forward',
                                                                                 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_no_dropout.init_weight': ( 'pann.html#cnn14_no_dropout.init_weight',
                                                                                     'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_no_specaug': ('pann.html#cnn14_no_specaug', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_no_specaug.__init__': ( 'pann.html#cnn14_no_specaug.__init__',
                                                                                  'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_no_specaug.forward': ( 'pann.html#cnn14_no_specaug.forward',
                                                                                 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn14_no_specaug.init_weight': ( 'pann.html#cnn14_no_specaug.init_weight',
                                                                                     'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn6': ('pann.html#cnn6', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn6.__init__': ('pann.html#cnn6.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn6.forward': ('pann.html#cnn6.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Cnn6.init_weight': ('pann.html#cnn6.init_weight', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ConvBlock': ('pann.html#convblock', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ConvBlock.__init__': ('pann.html#convblock.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ConvBlock.forward': ('pann.html#convblock.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ConvBlock.init_weight': ('pann.html#convblock.init_weight', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ConvBlock5x5': ('pann.html#convblock5x5', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ConvBlock5x5.__init__': ('pann.html#convblock5x5.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ConvBlock5x5.forward': ('pann.html#convblock5x5.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ConvBlock5x5.init_weight': ( 'pann.html#convblock5x5.init_weight',
                                                                                 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ConvPreWavBlock': ('pann.html#convprewavblock', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ConvPreWavBlock.__init__': ( 'pann.html#convprewavblock.__init__',
                                                                                 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ConvPreWavBlock.forward': ('pann.html#convprewavblock.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ConvPreWavBlock.init_weight': ( 'pann.html#convprewavblock.init_weight',
                                                                                    'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.DaiNet19': ('pann.html#dainet19', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.DaiNet19.__init__': ('pann.html#dainet19.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.DaiNet19.forward': ('pann.html#dainet19.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.DaiNet19.init_weight': ('pann.html#dainet19.init_weight', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.DaiNetResBlock': ('pann.html#dainetresblock', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.DaiNetResBlock.__init__': ('pann.html#dainetresblock.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.DaiNetResBlock.forward': ('pann.html#dainetresblock.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.DaiNetResBlock.init_weight': ( 'pann.html#dainetresblock.init_weight',
                                                                                   'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.InvertedResidual': ('pann.html#invertedresidual', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.InvertedResidual.__init__': ( 'pann.html#invertedresidual.__init__',
                                                                                  'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.InvertedResidual.forward': ( 'pann.html#invertedresidual.forward',
                                                                                 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.LeeNet11': ('pann.html#leenet11', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.LeeNet11.__init__': ('pann.html#leenet11.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.LeeNet11.forward': ('pann.html#leenet11.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.LeeNet11.init_weight': ('pann.html#leenet11.init_weight', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.LeeNet24': ('pann.html#leenet24', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.LeeNet24.__init__': ('pann.html#leenet24.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.LeeNet24.forward': ('pann.html#leenet24.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.LeeNet24.init_weight': ('pann.html#leenet24.init_weight', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.LeeNetConvBlock': ('pann.html#leenetconvblock', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.LeeNetConvBlock.__init__': ( 'pann.html#leenetconvblock.__init__',
                                                                                 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.LeeNetConvBlock.forward': ('pann.html#leenetconvblock.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.LeeNetConvBlock.init_weight': ( 'pann.html#leenetconvblock.init_weight',
                                                                                    'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.LeeNetConvBlock2': ('pann.html#leenetconvblock2', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.LeeNetConvBlock2.__init__': ( 'pann.html#leenetconvblock2.__init__',
                                                                                  'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.LeeNetConvBlock2.forward': ( 'pann.html#leenetconvblock2.forward',
                                                                                 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.LeeNetConvBlock2.init_weight': ( 'pann.html#leenetconvblock2.init_weight',
                                                                                     'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.MobileNetV1': ('pann.html#mobilenetv1', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.MobileNetV1.__init__': ('pann.html#mobilenetv1.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.MobileNetV1.forward': ('pann.html#mobilenetv1.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.MobileNetV1.init_weights': ( 'pann.html#mobilenetv1.init_weights',
                                                                                 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.MobileNetV2': ('pann.html#mobilenetv2', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.MobileNetV2.__init__': ('pann.html#mobilenetv2.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.MobileNetV2.forward': ('pann.html#mobilenetv2.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.MobileNetV2.init_weight': ('pann.html#mobilenetv2.init_weight', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Res1dNet31': ('pann.html#res1dnet31', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Res1dNet31.__init__': ('pann.html#res1dnet31.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Res1dNet31.forward': ('pann.html#res1dnet31.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Res1dNet31.init_weight': ('pann.html#res1dnet31.init_weight', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Res1dNet51': ('pann.html#res1dnet51', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Res1dNet51.__init__': ('pann.html#res1dnet51.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Res1dNet51.forward': ('pann.html#res1dnet51.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Res1dNet51.init_weight': ('pann.html#res1dnet51.init_weight', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ResNet22': ('pann.html#resnet22', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ResNet22.__init__': ('pann.html#resnet22.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ResNet22.forward': ('pann.html#resnet22.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ResNet22.init_weights': ('pann.html#resnet22.init_weights', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ResNet38': ('pann.html#resnet38', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ResNet38.__init__': ('pann.html#resnet38.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ResNet38.forward': ('pann.html#resnet38.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ResNet38.init_weights': ('pann.html#resnet38.init_weights', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ResNet54': ('pann.html#resnet54', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ResNet54.__init__': ('pann.html#resnet54.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ResNet54.forward': ('pann.html#resnet54.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.ResNet54.init_weights': ('pann.html#resnet54.init_weights', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Wavegram_Cnn14': ('pann.html#wavegram_cnn14', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Wavegram_Cnn14.__init__': ('pann.html#wavegram_cnn14.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Wavegram_Cnn14.forward': ('pann.html#wavegram_cnn14.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Wavegram_Cnn14.init_weight': ( 'pann.html#wavegram_cnn14.init_weight',
                                                                                   'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Wavegram_Logmel128_Cnn14': ( 'pann.html#wavegram_logmel128_cnn14',
                                                                                 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Wavegram_Logmel128_Cnn14.__init__': ( 'pann.html#wavegram_logmel128_cnn14.__init__',
                                                                                          'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Wavegram_Logmel128_Cnn14.forward': ( 'pann.html#wavegram_logmel128_cnn14.forward',
                                                                                         'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Wavegram_Logmel128_Cnn14.init_weight': ( 'pann.html#wavegram_logmel128_cnn14.init_weight',
                                                                                             'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Wavegram_Logmel_Cnn14': ('pann.html#wavegram_logmel_cnn14', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Wavegram_Logmel_Cnn14.__init__': ( 'pann.html#wavegram_logmel_cnn14.__init__',
                                                                                       'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Wavegram_Logmel_Cnn14.forward': ( 'pann.html#wavegram_logmel_cnn14.forward',
                                                                                      'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.Wavegram_Logmel_Cnn14.init_weight': ( 'pann.html#wavegram_logmel_cnn14.init_weight',
                                                                                          'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResNet': ('pann.html#_resnet', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResNet.__init__': ('pann.html#_resnet.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResNet._make_layer': ('pann.html#_resnet._make_layer', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResNet.forward': ('pann.html#_resnet.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResNetWav1d': ('pann.html#_resnetwav1d', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResNetWav1d.__init__': ('pann.html#_resnetwav1d.__init__', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResNetWav1d._make_layer': ( 'pann.html#_resnetwav1d._make_layer',
                                                                                 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResNetWav1d.forward': ('pann.html#_resnetwav1d.forward', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResnetBasicBlock': ('pann.html#_resnetbasicblock', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResnetBasicBlock.__init__': ( 'pann.html#_resnetbasicblock.__init__',
                                                                                   'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResnetBasicBlock.forward': ( 'pann.html#_resnetbasicblock.forward',
                                                                                  'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResnetBasicBlock.init_weights': ( 'pann.html#_resnetbasicblock.init_weights',
                                                                                       'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResnetBasicBlockWav1d': ('pann.html#_resnetbasicblockwav1d', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResnetBasicBlockWav1d.__init__': ( 'pann.html#_resnetbasicblockwav1d.__init__',
                                                                                        'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResnetBasicBlockWav1d.forward': ( 'pann.html#_resnetbasicblockwav1d.forward',
                                                                                       'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResnetBasicBlockWav1d.init_weights': ( 'pann.html#_resnetbasicblockwav1d.init_weights',
                                                                                            'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResnetBottleneck': ('pann.html#_resnetbottleneck', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResnetBottleneck.__init__': ( 'pann.html#_resnetbottleneck.__init__',
                                                                                   'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResnetBottleneck.forward': ( 'pann.html#_resnetbottleneck.forward',
                                                                                  'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._ResnetBottleneck.init_weights': ( 'pann.html#_resnetbottleneck.init_weights',
                                                                                       'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._resnet_conv1x1': ('pann.html#_resnet_conv1x1', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._resnet_conv1x1_wav1d': ('pann.html#_resnet_conv1x1_wav1d', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._resnet_conv3x1_wav1d': ('pann.html#_resnet_conv3x1_wav1d', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann._resnet_conv3x3': ('pann.html#_resnet_conv3x3', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.init_bn': ('pann.html#init_bn', 'fad_pytorch/pann.py'),
                                  'fad_pytorch.pann.init_layer': ('pann.html#init_layer', 'fad_pytorch/pann.py')},
            'fad_pytorch.pann_pytorch_utils': {},
            'fad_pytorch.sqrtm': { 'fad_pytorch.sqrtm.MatrixSquareRoot_li': ('sqrtm.html#matrixsquareroot_li', 'fad_pytorch/sqrtm.py'),
                                   'fad_pytorch.sqrtm.MatrixSquareRoot_li.backward': ( 'sqrtm.html#matrixsquareroot_li.backward',
                                                                                       'fad_pytorch/sqrtm.py'),
                                   'fad_pytorch.sqrtm.MatrixSquareRoot_li.forward': ( 'sqrtm.html#matrixsquareroot_li.forward',
                                                                                      'fad_pytorch/sqrtm.py'),
                                   'fad_pytorch.sqrtm.compute_error': ('sqrtm.html#compute_error', 'fad_pytorch/sqrtm.py'),
                                   'fad_pytorch.sqrtm.sqrt_newton_schulz': ('sqrtm.html#sqrt_newton_schulz', 'fad_pytorch/sqrtm.py'),
                                   'fad_pytorch.sqrtm.sqrt_newton_schulz_autograd': ( 'sqrtm.html#sqrt_newton_schulz_autograd',
                                                                                      'fad_pytorch/sqrtm.py'),
                                   'fad_pytorch.sqrtm.sqrtm': ('sqrtm.html#sqrtm', 'fad_pytorch/sqrtm.py')}}}
